@@ -25,10 +25,6 @@ from diskimage_builder.block_device.utils import exec_sudo
 logger = logging.getLogger(__name__)
 
 
-# There is the need that filesystem labels are unique:
-# if not the boot and / or mount (with LABEL=) might fail.
-file_system_labels = set()
-
 # There is the need to check the length of the label of
 # the filesystem.  The maximum length depends on the used filesystem.
 # This map provides information about the maximum label length.
@@ -43,9 +39,9 @@ file_system_max_label_length = {
 
 class FilesystemNode(NodeBase):
 
-    def __init__(self, config):
-        logger.debug("Create filesystem object; config [%s]" % config)
-        super(FilesystemNode, self).__init__(config['name'])
+    def __init__(self, config, state):
+        logger.debug("Create filesystem object; config [%s]", config)
+        super(FilesystemNode, self).__init__(config['name'], state)
 
         # Parameter check (mandatory)
         for pname in ['base', 'type']:
@@ -69,24 +65,29 @@ class FilesystemNode(NodeBase):
                            "file system - using [img-rootfs] instead")
             self.label = "img-rootfs"
 
-        if self.label in file_system_labels:
-            raise BlockDeviceSetupException(
-                "File system label [%s] used more than once" % self.label)
-        file_system_labels.add(self.label)
+        # ensure we don't already have a fs with this label ... they
+        # all must be unique.
+        if 'fs_labels' in self.state:
+            if self.label in self.state['fs_labels']:
+                raise BlockDeviceSetupException(
+                    "File system label [%s] used more than once" % self.label)
+            self.state['fs_labels'].append(self.label)
+        else:
+            self.state['fs_labels'] = [self.label]
 
         if self.type in file_system_max_label_length:
             if file_system_max_label_length[self.type] < len(self.label):
                 raise BlockDeviceSetupException(
                     "Label [{label}] too long for filesystem [{type}]: "
-                    " [{len}] > [{max_len}]".format({
+                    "{len} > {max_len}".format(**{
                         'label': self.label,
                         'type': self.type,
                         'len': len(self.label),
-                        'max': file_system_max_label_length[self.type]}))
+                        'max_len': file_system_max_label_length[self.type]}))
         else:
             logger.warning("Length of label [%s] cannot be checked for "
-                           "filesystem [%s]: unknown max length" %
-                           (self.label, self.type))
+                           "filesystem [%s]: unknown max length",
+                           self.label, self.type)
             logger.warning("Continue - but this might lead to an error")
 
         if self.opts is not None:
@@ -95,16 +96,14 @@ class FilesystemNode(NodeBase):
         if self.uuid is None:
             self.uuid = str(uuid.uuid4())
 
-        logger.debug("Filesystem created [%s]" % self)
+        logger.debug("Filesystem created [%s]", self)
 
     def get_edges(self):
         edge_from = [self.base]
         edge_to = []
         return (edge_from, edge_to)
 
-    def create(self, result, rollback):
-        logger.info("create called; result [%s]" % result)
-
+    def create(self):
         cmd = ["mkfs"]
 
         cmd.extend(['-t', self.type])
@@ -117,23 +116,23 @@ class FilesystemNode(NodeBase):
         elif self.type == 'xfs':
             cmd.extend(['-m', "uuid=%s" % self.uuid])
         else:
-            logger.warning("UUID will not be written for fs type [%s]"
-                           % self.type)
+            logger.warning("UUID will not be written for fs type [%s]",
+                           self.type)
 
         if self.type in ('ext2', 'ext3', 'ext4', 'xfs'):
             cmd.append('-q')
 
-        if 'blockdev' not in result:
-            result['blockdev'] = {}
-        device = result['blockdev'][self.base]['device']
+        if 'blockdev' not in self.state:
+            self.state['blockdev'] = {}
+        device = self.state['blockdev'][self.base]['device']
         cmd.append(device)
 
-        logger.debug("Creating fs command [%s]" % (cmd))
+        logger.debug("Creating fs command [%s]", cmd)
         exec_sudo(cmd)
 
-        if 'filesys' not in result:
-            result['filesys'] = {}
-        result['filesys'][self.name] \
+        if 'filesys' not in self.state:
+            self.state['filesys'] = {}
+        self.state['filesys'][self.name] \
             = {'uuid': self.uuid, 'label': self.label,
                'fstype': self.type, 'opts': self.opts,
                'device': device}
@@ -146,10 +145,10 @@ class Mkfs(PluginBase):
     systems.
     """
 
-    def __init__(self, config, defaults):
+    def __init__(self, config, defaults, state):
         super(Mkfs, self).__init__()
         self.filesystems = {}
-        fs = FilesystemNode(config)
+        fs = FilesystemNode(config, state)
         self.filesystems[fs.get_name()] = fs
 
     def get_nodes(self):
